#include "rainbowModule.h"
#include "geometry.h"


//-------------------------------------------------
void hsv2rgb( float h, float s, float v, float &r, float &g, float &b ) ;
void hsv2rgb( float h, float s, float v, float &r, float &g, float &b ) {
	h *= 360.f;
	int i;
	float f, p, q, t;
	if( s == 0 ) {
		// achromatic (grey)
		r = g = b = v;
		return;
	}
	h /= 60.f;			// sector 0 to 5
	i = (int)floor( h );
	f = h - i;			// factorial part of h
	p = v * ( 1.f - s );
	q = v * ( 1.f - s * f );
	t = v * ( 1.f - s * ( 1.f - f ) );
	switch( i ) {
		case 0: r = v; g = t; b = p; break;
		case 1: r = q; g = v; b = p; break;
		case 2: r = p; g = v; b = t; break;
		case 3: r = p; g = q; b = v; break;
		case 4: r = t; g = p; b = v; break;
		default: r = v; g = p; b = q; break;
	}
}




//-------------------------------------------------
rainbowModule::rainbowModule(){
	
}

//-------------------------------------------------	
void rainbowModule::setup(){
	bgColor.set(255,255,255);
	bDone 			= false;
	reset();
}

//-------------------------------------------------
void rainbowModule::reset(){
	pct 	= 0;
	bDone 	= false;
	
	pts[0].set(0,ofRandomuf()*480,0);
	pts[3].set(640,ofRandomuf()*480,0);
	
	pts[1] = pts[0] + 0.33f * (pts[3]-pts[0]);
	float angle = ofRandomuf() * TWO_PI;
	ofVec3f angTang(cos(angle), sin(angle), 0);
	pts[1] += angTang * (100 + 100 * ofRandomuf());
	
	pts[1].y = MAX(MIN(pts[1].y, 480), 0);
	pts[2] = pts[1] + 0.50f * (pts[3]-pts[1]);
	angle = ofRandomuf() * TWO_PI;
	angTang.set(cos(angle), sin(angle), 0);
	pts[2] += angTang * (50 + 100 * ofRandomuf());
}


//-------------------------------------------------
void rainbowModule::update(){
	pct += 0.01f;
	if ((int)(pct * 100) % 5 == 0) addPoint(ofVec3f(pct * 640, 200,0));
	if (pct > 1) bDone = true;
}

//-------------------------------------------------
void rainbowModule::draw(){
	ofSetColor(0x00ff00);
	
	ofVec3f pt = getBezierPt(pts, pct);
	
	ofVec3f color;
	hsv2rgb(pct, 1,1,color.x, color.y, color.z);
	
	ofSetColor(color.x*255, color.y*255, color.z*255);
	ofRect(pt.x, pt.y,20,20);
	
	
	glBegin(GL_LINE_STRIP);
	for (int i = 0; i < 4; i++){
		glVertex2f(pts[i].x, pts[i].y);
	}
	glEnd();
	

}

//-------------------------------------------------
bool rainbowModule::bAmIdone(){
	return bDone;
}

//-------------------------------------------------
ofVec3f rainbowModule::getStartPosition(){
	return ofVec3f(0,240,0);
}

//-------------------------------------------------
float rainbowModule::getStartAngle(){
	return 0;
}
		
		

